@@ -26,6 +26,12 @@ pub enum Error {
 
     #[error("bincode: {0}")]
     Bincode(#[from] bincode::Error),
+
+    #[error("Jupiter API: {0}")]
+    JupiterApi(String),
+
+    #[error("serde_json: {0}")]
+    SerdeJson(#[from] serde_json::Error),
 }
 
 /// Generic response with timing information
@@ -96,6 +102,21 @@ pub struct Swap {
 /// Hashmap of possible swap routes from input mint to an array of output mints
 pub type RouteMap = HashMap<Pubkey, Vec<Pubkey>>;
 
+fn maybe_jupiter_api_error<T>(value: serde_json::Value) -> Result<T>
+where
+    T: serde::de::DeserializeOwned,
+{
+    #[derive(Deserialize)]
+    struct ErrorResponse {
+        message: String,
+    }
+    if let Ok(ErrorResponse { message }) = serde_json::from_value::<ErrorResponse>(value.clone()) {
+        Err(Error::JupiterApi(message))
+    } else {
+        serde_json::from_value(value).map_err(|err| err.into())
+    }
+}
+
 /// Get simple price for a given input mint, output mint and amount
 pub async fn price(
     input_mint: Pubkey,
@@ -106,11 +127,7 @@ pub async fn price(
         "https://quote-api.jup.ag/v1/price?inputMint={}&outputMint={}&amount={}",
         input_mint, output_mint, ui_amount,
     );
-    reqwest::get(url)
-        .await?
-        .json()
-        .await
-        .map_err(|err| err.into())
+    maybe_jupiter_api_error(reqwest::get(url).await?.json().await?)
 }
 
 /// Get quote for a given input mint, output mint and amount
@@ -136,15 +153,22 @@ pub async fn quote(
             .unwrap_or_default(),
     );
 
-    reqwest::get(url)
-        .await?
-        .json()
-        .await
-        .map_err(|err| err.into())
+    maybe_jupiter_api_error(reqwest::get(url).await?.json().await?)
 }
 
-/// Get swap serialized transactions for a quote
-pub async fn swap(route: Quote, user_public_key: Pubkey) -> Result<Swap> {
+#[derive(Default)]
+pub struct SwapConfig {
+    pub wrap_unwrap_sol: Option<bool>,
+    pub fee_account: Option<Pubkey>,
+    pub token_ledger: Option<Pubkey>,
+}
+
+/// Get swap serialized transactions for a quote using `SwapConfig` defaults
+pub async fn swap_with_config(
+    route: Quote,
+    user_public_key: Pubkey,
+    swap_config: SwapConfig,
+) -> Result<Swap> {
     let url = "https://quote-api.jup.ag/v1/swap";
 
     #[derive(Debug, Serialize)]
@@ -152,7 +176,7 @@ pub async fn swap(route: Quote, user_public_key: Pubkey) -> Result<Swap> {
     #[allow(non_snake_case)]
     struct SwapRequest {
         route: Quote,
-        wrap_unwrap_SOL: bool,
+        wrap_unwrap_SOL: Option<bool>,
         fee_account: Option<String>,
         token_ledger: Option<String>,
         #[serde(with = "field_as_string")]
@@ -169,21 +193,23 @@ pub async fn swap(route: Quote, user_public_key: Pubkey) -> Result<Swap> {
 
     let request = SwapRequest {
         route,
-        wrap_unwrap_SOL: true,
-        fee_account: None,
-        token_ledger: None,
+        wrap_unwrap_SOL: swap_config.wrap_unwrap_sol,
+        fee_account: swap_config.fee_account.map(|x| x.to_string()),
+        token_ledger: swap_config.token_ledger.map(|x| x.to_string()),
         user_public_key,
     };
 
-    let response = reqwest::Client::builder()
-        .build()?
-        .post(url)
-        .json(&request)
-        .send()
-        .await?
-        .error_for_status()?
-        .json::<SwapResponse>()
-        .await?;
+    let response = maybe_jupiter_api_error::<SwapResponse>(
+        reqwest::Client::builder()
+            .build()?
+            .post(url)
+            .json(&request)
+            .send()
+            .await?
+            .error_for_status()?
+            .json()
+            .await?,
+    )?;
 
     fn decode(base64_transaction: String) -> Result<Transaction> {
         bincode::deserialize(&base64::decode(base64_transaction)?).map_err(|err| err.into())
@@ -196,6 +222,11 @@ pub async fn swap(route: Quote, user_public_key: Pubkey) -> Result<Swap> {
     })
 }
 
+/// Get swap serialized transactions for a quote
+pub async fn swap(route: Quote, user_public_key: Pubkey) -> Result<Swap> {
+    swap_with_config(route, user_public_key, SwapConfig::default()).await
+}
+
 /// Returns a hash map, input mint as key and an array of valid output mint as values
 pub async fn route_map(only_direct_routes: bool) -> Result<RouteMap> {
     let url = format!(
